@@ -542,4 +542,57 @@ public class ExprParserTest {
         expr = new ExprParser("dd ** .X", false).parse();
         assertEquals("$.dd**.X", ExprUnparser.exprToString(expr));
     }
+
+    /**
+     * Fix for Bug#95503 (29821029), Operator IN not mapping consistently to the right X Plugin operation.
+     */
+    @Test
+    public void testBug95503() {
+        Expr expr;
+
+        expr = new ExprParser("field IN (1, 2, 3)").parse();
+        assertEquals(Expr.Type.OPERATOR, expr.getType());
+        assertEquals("in", expr.getOperator().getName());
+
+        expr = new ExprParser("field IN value").parse();
+        assertEquals(Expr.Type.OPERATOR, expr.getType());
+        assertEquals("cont_in", expr.getOperator().getName());
+
+        expr = new ExprParser("field IN 123 + 456").parse();
+        assertEquals(Expr.Type.OPERATOR, expr.getType());
+        assertEquals("cont_in", expr.getOperator().getName());
+
+        expr = new ExprParser("field IN [1, 2, 3]").parse();
+        assertEquals(Expr.Type.OPERATOR, expr.getType());
+        assertEquals("cont_in", expr.getOperator().getName());
+
+        expr = new ExprParser("field IN {\"foo\": \"bar\"}").parse();
+        assertEquals(Expr.Type.OPERATOR, expr.getType());
+        assertEquals("cont_in", expr.getOperator().getName());
+    }
+
+    @Test
+    public void testOverlaps() {
+        Expr expr = new ExprParser("[1, 2, 3] OVERLAPS $.list", false).parse();
+        assertEquals("[1, 2, 3] overlaps $.list", ExprUnparser.exprToString(expr));
+
+        expr = new ExprParser("$.list OVERLAPS [4]", false).parse();
+        assertEquals("$.list overlaps [4]", ExprUnparser.exprToString(expr));
+
+        expr = new ExprParser("[1, 2, 3] NOT OVERLAPS $.list", false).parse();
+        assertEquals("[1, 2, 3] not overlaps $.list", ExprUnparser.exprToString(expr));
+
+        expr = new ExprParser("$.list NOT OVERLAPS [4]", false).parse();
+        assertEquals("$.list not overlaps [4]", ExprUnparser.exprToString(expr));
+    }
+
+    @Test
+    public void testOverlapsInProjection() {
+        List<Projection> proj;
+
+        proj = new ExprParser("$.`overlaps` as `overlaps`").parseDocumentProjection();
+        assertEquals(1, proj.size());
+        assertTrue(proj.get(0).hasAlias());
+        assertEquals("overlaps", proj.get(0).getAlias());
+    }
 }
