@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2020, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License, version 2.0, as published by the
@@ -80,7 +80,6 @@ public abstract class BaseTestCase extends TestCase {
     protected boolean DISABLED_testBug65503 = true; // TODO disabled for unknown reason
     protected boolean DISABLED_testContention = true; // TODO disabled for unknown reason
     protected boolean DISABLED_testBug3620new = true; // TODO this test is working in c/J 5.1 but fails here; disabled for later analysis
-    protected boolean DISABLED_testBug5874 = true; // TODO this test is working in c/J 5.1 but fails here; disabled for later analysis
 
     /**
      * JDBC URL, initialized from com.mysql.cj.testsuite.url system property, or defaults to jdbc:mysql:///test and its connection URL.
@@ -666,7 +665,7 @@ public abstract class BaseTestCase extends TestCase {
 
         this.serverVersion = ((JdbcConnection) this.conn).getServerVersion();
 
-        this.stmt = this.conn.createStatement();
+        this.stmt = this.conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
 
         try {
             if (dbUrl.indexOf("mysql") != -1) {
@@ -823,7 +822,7 @@ public abstract class BaseTestCase extends TestCase {
      * @throws SQLException
      *             if an error occurs.
      */
-    protected boolean versionMeetsMinimum(int major, int minor, int subminor) throws SQLException {
+    public boolean versionMeetsMinimum(int major, int minor, int subminor) throws SQLException {
         return (((JdbcConnection) this.conn).getSession().versionMeetsMinimum(major, minor, subminor));
     }
 
