@@ -42,6 +42,7 @@ import com.mysql.cj.xdevapi.Collection;
 import com.mysql.cj.xdevapi.DocResult;
 import com.mysql.cj.xdevapi.JsonNumber;
 import com.mysql.cj.xdevapi.RemoveStatement;
+import com.mysql.cj.xdevapi.RemoveStatementImpl;
 import com.mysql.cj.xdevapi.Result;
 import com.mysql.cj.xdevapi.Session;
 import com.mysql.cj.xdevapi.SessionFactory;
@@ -398,4 +399,22 @@ public class CollectionRemoveTest extends BaseCollectionTestCase {
             assertEquals(v, ((JsonNumber) docRes.next().get("ord")).getInteger().intValue());
         }
     }
+
+    @Test
+    @SuppressWarnings("deprecation")
+    public void testDeprecateWhere() throws Exception {
+        if (!this.isSetForXTests) {
+            return;
+        }
+
+        this.collection.add("{\"_id\":\"1\", \"ord\": 1}", "{\"_id\":\"2\", \"ord\": 2}", "{\"_id\":\"3\", \"ord\": 3}", "{\"_id\":\"4\", \"ord\": 4}",
+                "{\"_id\":\"5\", \"ord\": 5}", "{\"_id\":\"6\", \"ord\": 6}", "{\"_id\":\"7\", \"ord\": 7}", "{\"_id\":\"8\", \"ord\": 8}").execute();
+
+        RemoveStatement testRemove = this.collection.remove("$.ord <= 2");
+
+        assertTrue(testRemove.getClass().getMethod("where", String.class).isAnnotationPresent(Deprecated.class));
+
+        assertEquals(2, testRemove.execute().getAffectedItemsCount());
+        assertEquals(4, ((RemoveStatementImpl) testRemove).where("$.ord > 4").execute().getAffectedItemsCount());
+    }
 }
