@@ -42,6 +42,7 @@ import java.lang.ref.WeakReference;
 import java.lang.reflect.Field;
 import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
@@ -49,6 +50,8 @@ import java.util.Random;
 import java.util.Set;
 import java.util.concurrent.BlockingQueue;
 import java.util.concurrent.Callable;
+import java.util.stream.Collectors;
+import java.util.stream.IntStream;
 
 import org.junit.After;
 import org.junit.Before;
@@ -77,10 +80,13 @@ import com.mysql.cj.xdevapi.SelectStatement;
 import com.mysql.cj.xdevapi.Session;
 import com.mysql.cj.xdevapi.SessionFactory;
 import com.mysql.cj.xdevapi.SessionImpl;
+import com.mysql.cj.xdevapi.SqlMultiResult;
 import com.mysql.cj.xdevapi.SqlResult;
 import com.mysql.cj.xdevapi.SqlStatement;
 import com.mysql.cj.xdevapi.XDevAPIError;
 
+import testsuite.UnreliableSocketFactory;
+
 public class SessionTest extends DevApiBaseTestCase {
     @Before
     public void setupSessionTest() {
@@ -1104,7 +1110,7 @@ public class SessionTest extends DevApiBaseTestCase {
             Session sess2 = cli1.getSession(); // new connection #2
             sess2.sql("SELECT 1").execute();
 
-            assertThrows(CJCommunicationsException.class, "Cannot read packet header", () -> cli2.getSession());
+            assertThrows(CJCommunicationsException.class, "Unable to connect to any of the target hosts\\.", cli2::getSession);
 
             cli1.close();
             cli2.close();
@@ -1129,12 +1135,17 @@ public class SessionTest extends DevApiBaseTestCase {
                 this.fact.getSession(url);
                 fail("The named-pipe connection attempt must fail with " + (path != null ? path : "default") + " path.");
             } catch (Exception e) {
-                if (e instanceof CJCommunicationsException && e.getCause() != null && e.getCause() instanceof FileNotFoundException
-                        && ((path == null ? "\\\\.\\pipe\\MySQL" : path) + " (The system cannot find the file specified)").equals(e.getCause().getMessage())) {
-                    continue;
-                } else if (e instanceof XProtocolError && "ASSERTION FAILED: Unknown message type: 10 (server messages mapping: null)".equals(e.getMessage())) {
-                    // if named pipes are enabled on server then we expect this error because the pipe is bound to legacy protocol
-                    continue;
+                Throwable cause = e.getCause();
+                if (cause != null) {
+                    if (cause instanceof CJCommunicationsException && cause.getCause() != null && cause.getCause() instanceof FileNotFoundException
+                            && ((path == null ? "\\\\.\\pipe\\MySQL" : path) + " (The system cannot find the file specified)")
+                                    .equals(cause.getCause().getMessage())) {
+                        continue;
+                    } else if (cause instanceof XProtocolError
+                            && "ASSERTION FAILED: Unknown message type: 10 (server messages mapping: null)".equals(cause.getMessage())) {
+                        // if named pipes are enabled on server then we expect this error because the pipe is bound to legacy protocol
+                        continue;
+                    }
                 }
                 throw e;
             }
@@ -1564,4 +1575,433 @@ public class SessionTest extends DevApiBaseTestCase {
             this.schema.dropCollection("testPrepStmtPooling");
         }
     }
+
+    @Test
+    public void testBug23721537() throws Exception {
+        if (!this.isSetForXTests) {
+            return;
+        }
+        try {
+            sqlUpdate("drop table if exists testBug23721537");
+            sqlUpdate("create table testBug23721537 (id int, name varchar(20) not null)");
+            sqlUpdate("insert into testBug23721537 values (0, 'a')");
+
+            this.session.sql("drop procedure if exists newproc").execute();
+            this.session.sql(
+                    "create procedure newproc (in p1 int,in p2 char(20)) begin select 1; update testBug23721537 set name='b' where id=0; select 2; select 3; end;")
+                    .execute();
+
+            /* sync execution */
+            SqlResult res1 = this.session.sql("call newproc(?,?)").bind(10).bind("X").execute();
+
+            assertTrue(res1 instanceof SqlMultiResult);
+            assertTrue(res1.hasData());
+            assertTrue(res1.hasNext());
+            Row r = res1.next();
+            assertEquals(1, r.getInt(0));
+            assertFalse(res1.hasNext());
+
+            SqlResult res2 = this.session.sql("select 10").execute(); // res1 should finish streaming here
+
+            assertTrue(res1.nextResult());
+            assertTrue(res1.hasData());
+            assertTrue(res1.hasNext());
+            r = res1.next();
+            assertEquals(2, r.getInt(0));
+            assertFalse(res1.hasNext());
+
+            assertTrue(res1.nextResult());
+            assertTrue(res1.hasData());
+            assertTrue(res1.hasNext());
+            r = res1.next();
+            assertEquals(3, r.getInt(0));
+            assertFalse(res1.hasNext());
+
+            assertFalse(res1.nextResult());
+
+            //
+            assertTrue(res2.hasData());
+            assertTrue(res2.hasNext());
+            r = res2.next();
+            assertEquals(10, r.getInt(0));
+            assertFalse(res2.hasNext());
+            assertFalse(res2.nextResult());
+
+            /* async execution */
+            res1 = this.session.sql("call newproc(?,?)").bind(10).bind("X").executeAsync().get();
+
+            assertTrue(res1.hasData());
+            r = res1.next();
+            assertEquals(1, r.getInt(0));
+            assertFalse(res1.hasNext());
+
+            res2 = this.session.sql("select 10").executeAsync().get(); // res1 should finish streaming here
+
+            assertTrue(res1.nextResult());
+            assertTrue(res1.hasData());
+            assertTrue(res1.hasNext());
+            r = res1.next();
+            assertEquals(2, r.getInt(0));
+            assertFalse(res1.hasNext());
+
+            assertTrue(res1.nextResult());
+            assertTrue(res1.hasData());
+            assertTrue(res1.hasNext());
+            r = res1.next();
+            assertEquals(3, r.getInt(0));
+            assertFalse(res1.hasNext());
+
+            assertFalse(res1.nextResult());
+
+            //
+            assertTrue(res2.hasData());
+            assertTrue(res2.hasNext());
+            r = res2.next();
+            assertEquals(10, r.getInt(0));
+            assertFalse(res2.hasNext());
+            assertFalse(res2.nextResult());
+
+        } finally {
+            sqlUpdate("drop table if exists testBug23721537");
+            this.session.sql("drop procedure if exists newproc").execute();
+        }
+    }
+
+    @Test
+    public void basicSessionFailoverRandomSort() {
+        if (!this.isSetForXTests) {
+            return;
+        }
+        UnreliableSocketFactory.flushAllStaticData();
+
+        final String testUriPattern = "mysqlx://%s:%s@[%s]/%s?" + PropertyKey.xdevapiConnectTimeout.getKeyName() + "=100&"
+                + PropertyKey.socketFactory.getKeyName() + "=" + UnreliableSocketFactory.class.getName();
+        String testHosts = IntStream.range(1, 6).mapToObj(i -> "host" + i).peek(h -> UnreliableSocketFactory.mapHost(h, getTestHost()))
+                .map(h -> h + ":" + getTestPort()).collect(Collectors.joining(","));
+        String testUri = String.format(testUriPattern, getTestUser(), getTestPassword(), testHosts, getTestDatabase());
+
+        Set<String> downHosts = new HashSet<>();
+        for (int i = 1; i <= 5; i++) {
+            Session testSession = this.fact.getSession(testUri);
+            assertTrue(UnreliableSocketFactory.isConnected());
+            testSession.close();
+
+            String lastHost = UnreliableSocketFactory.getHostFromLastConnection();
+            for (String h : downHosts) {
+                assertNotEquals(h, lastHost);
+            }
+            downHosts.add(lastHost);
+            UnreliableSocketFactory.downHost(lastHost.substring(1));
+        }
+
+        // None of the hosts is available by now.
+        assertThrows(CJCommunicationsException.class, "Unable to connect to any of the target hosts\\.", () -> {
+            this.fact.getSession(testUri);
+            return null;
+        });
+
+        UnreliableSocketFactory.dontDownHost("host3");
+
+        UnreliableSocketFactory.flushConnectionAttempts();
+        Session testSession = this.fact.getSession(testUri);
+        assertTrue(UnreliableSocketFactory.isConnected());
+        testSession.close();
+        assertEquals(UnreliableSocketFactory.getHostConnectedStatus("host3"), UnreliableSocketFactory.getHostFromLastConnection());
+    }
+
+    @Test
+    public void basicSessionFailoverByPriorities() {
+        if (!this.isSetForXTests) {
+            return;
+        }
+        UnreliableSocketFactory.flushAllStaticData();
+
+        final String testUriPattern = "mysqlx://%s:%s@[%s]/%s?" + PropertyKey.xdevapiConnectTimeout.getKeyName() + "=100&"
+                + PropertyKey.socketFactory.getKeyName() + "=" + UnreliableSocketFactory.class.getName();
+        String testHosts = IntStream.range(1, 6).mapToObj(i -> "host" + i).peek(h -> UnreliableSocketFactory.mapHost(h, getTestHost()))
+                .map(h -> "(address=" + h + ":" + getTestPort() + ",priority=%d)").collect(Collectors.joining(","));
+        String testUriPatternPriorities = String.format(testUriPattern, getTestUser(), getTestPassword(), testHosts, getTestDatabase());
+        String testUri = String.format(testUriPatternPriorities, 60, 80, 100, 20, 40);
+        int[] hostsOrder = new int[] { 3, 2, 1, 5, 4 };
+
+        for (int i = 0; i < hostsOrder.length; i++) {
+            int h = hostsOrder[i];
+
+            Session testSession = this.fact.getSession(testUri);
+            assertTrue(UnreliableSocketFactory.isConnected());
+            testSession.close();
+
+            String lastHost = UnreliableSocketFactory.getHostFromLastConnection();
+            assertEquals(UnreliableSocketFactory.getHostConnectedStatus("host" + h), lastHost);
+            List<String> connectionAttempts = UnreliableSocketFactory.getHostsFromAllConnections();
+            for (int a = 0; a < i; a++) {
+                assertEquals(UnreliableSocketFactory.getHostFailedStatus("host" + hostsOrder[a]), connectionAttempts.get(a));
+            }
+            UnreliableSocketFactory.downHost("host" + h);
+            UnreliableSocketFactory.flushConnectionAttempts();
+        }
+
+        // None of the hosts is available by now.
+        assertThrows(CJCommunicationsException.class, "Unable to connect to any of the target hosts\\.", () -> {
+            this.fact.getSession(testUri);
+            return null;
+        });
+
+        UnreliableSocketFactory.dontDownHost("host" + hostsOrder[1]);
+
+        UnreliableSocketFactory.flushConnectionAttempts();
+        Session testSession = this.fact.getSession(testUri);
+        assertTrue(UnreliableSocketFactory.isConnected());
+        testSession.close();
+        List<String> connectionAttempts = UnreliableSocketFactory.getHostsFromAllConnections();
+        assertEquals(2, connectionAttempts.size());
+        assertEquals(UnreliableSocketFactory.getHostFailedStatus("host" + hostsOrder[0]), connectionAttempts.get(0));
+        assertEquals(UnreliableSocketFactory.getHostConnectedStatus("host" + hostsOrder[1]), connectionAttempts.get(1));
+    }
+
+    @Test
+    public void pooledSessionFailoverRandomSortAndPooling() {
+        if (!this.isSetForXTests) {
+            return;
+        }
+        UnreliableSocketFactory.flushAllStaticData();
+
+        final String testUriPattern = "mysqlx://%s:%s@[%s]/%s?" + PropertyKey.xdevapiConnectTimeout.getKeyName() + "=100&"
+                + PropertyKey.socketFactory.getKeyName() + "=" + UnreliableSocketFactory.class.getName();
+        String testHosts = IntStream.range(1, 6).mapToObj(i -> "host" + i).peek(h -> UnreliableSocketFactory.mapHost(h, getTestHost()))
+                .map(h -> h + ":" + getTestPort()).collect(Collectors.joining(","));
+        String testUri = String.format(testUriPattern, getTestUser(), getTestPassword(), testHosts, getTestDatabase());
+
+        final ClientFactory cf = new ClientFactory();
+        Client client = cf.getClient(testUri, "{\"pooling\" : {\"enabled\" : true, \"maxSize\" : 10} }");
+
+        Set<String> downHosts = new HashSet<>();
+        for (int i = 1; i <= 5; i++) {
+            Session testSession = client.getSession();
+            assertTrue(UnreliableSocketFactory.isConnected());
+            testSession.close(); // Pool this connection.
+
+            String lastHost = UnreliableSocketFactory.getHostFromLastConnection();
+            for (String h : downHosts) {
+                assertNotEquals(h, lastHost);
+            }
+            downHosts.add(lastHost);
+            UnreliableSocketFactory.downHost(lastHost.substring(1));
+            UnreliableSocketFactory.flushConnectionAttempts();
+        }
+
+        // None of the hosts is available by now.
+        assertThrows(CJCommunicationsException.class, "Unable to connect to any of the target hosts\\.", () -> {
+            client.getSession();
+            return null;
+        });
+
+        UnreliableSocketFactory.dontDownHost("host3");
+
+        UnreliableSocketFactory.flushConnectionAttempts();
+        Session testSession = client.getSession();
+        assertTrue(UnreliableSocketFactory.isConnected());
+        testSession.close();
+        assertEquals(UnreliableSocketFactory.getHostConnectedStatus("host3"), UnreliableSocketFactory.getHostFromLastConnection());
+
+        UnreliableSocketFactory.flushConnectionAttempts();
+        Session testSession1 = client.getSession(); // Pick previous connection from the pool. Doesn't count as new connections.
+        assertEquals(0, UnreliableSocketFactory.getHostsFromAllConnections().size());
+        Session testSession2 = client.getSession(); // Create a new connection.
+        assertTrue(UnreliableSocketFactory.isConnected());
+        testSession1.close();
+        testSession2.close();
+        assertEquals(UnreliableSocketFactory.getHostConnectedStatus("host3"), UnreliableSocketFactory.getHostFromLastConnection());
+
+        client.close();
+    }
+
+    @Test
+    public void pooledSessionFailoverRandomSortAndNoPooling() {
+        if (!this.isSetForXTests) {
+            return;
+        }
+        UnreliableSocketFactory.flushAllStaticData();
+
+        final String testUriPattern = "mysqlx://%s:%s@[%s]/%s?" + PropertyKey.xdevapiConnectTimeout.getKeyName() + "=100&"
+                + PropertyKey.socketFactory.getKeyName() + "=" + UnreliableSocketFactory.class.getName();
+        String testHosts = IntStream.range(1, 6).mapToObj(i -> "host" + i).peek(h -> UnreliableSocketFactory.mapHost(h, getTestHost()))
+                .map(h -> h + ":" + getTestPort()).collect(Collectors.joining(","));
+        String testUri = String.format(testUriPattern, getTestUser(), getTestPassword(), testHosts, getTestDatabase());
+
+        final ClientFactory cf = new ClientFactory();
+        Client client = cf.getClient(testUri, "{\"pooling\" : {\"enabled\" : true, \"maxSize\" : 10} }");
+
+        Set<String> downHosts = new HashSet<>();
+        for (int i = 1; i <= 5; i++) {
+            client.getSession(); // Don't pool this connection.
+            assertTrue(UnreliableSocketFactory.isConnected());
+
+            String lastHost = UnreliableSocketFactory.getHostFromLastConnection();
+            for (String h : downHosts) {
+                assertNotEquals(h, lastHost);
+            }
+            downHosts.add(lastHost);
+            UnreliableSocketFactory.downHost(lastHost.substring(1));
+            UnreliableSocketFactory.flushConnectionAttempts();
+        }
+
+        // None of the hosts is available by now.
+        assertThrows(CJCommunicationsException.class, "Unable to connect to any of the target hosts\\.", () -> {
+            client.getSession();
+            return null;
+        });
+
+        UnreliableSocketFactory.dontDownHost("host3");
+
+        UnreliableSocketFactory.flushConnectionAttempts();
+        client.getSession();
+        assertTrue(UnreliableSocketFactory.isConnected());
+        assertEquals(UnreliableSocketFactory.getHostConnectedStatus("host3"), UnreliableSocketFactory.getHostFromLastConnection());
+
+        UnreliableSocketFactory.flushConnectionAttempts();
+        client.getSession();
+        assertTrue(UnreliableSocketFactory.isConnected());
+        assertEquals(UnreliableSocketFactory.getHostConnectedStatus("host3"), UnreliableSocketFactory.getHostFromLastConnection());
+
+        client.close();
+    }
+
+    @Test
+    public void pooledSessionFailoverByPrioritiesAndPooling() {
+        if (!this.isSetForXTests) {
+            return;
+        }
+        UnreliableSocketFactory.flushAllStaticData();
+
+        final String testUriPattern = "mysqlx://%s:%s@[%s]/%s?" + PropertyKey.xdevapiConnectTimeout.getKeyName() + "=100&"
+                + PropertyKey.socketFactory.getKeyName() + "=" + UnreliableSocketFactory.class.getName();
+        String testHosts = IntStream.range(1, 6).mapToObj(i -> "host" + i).peek(h -> UnreliableSocketFactory.mapHost(h, getTestHost()))
+                .map(h -> "(address=" + h + ":" + getTestPort() + ",priority=%d)").collect(Collectors.joining(","));
+        String testUriPatternPriorities = String.format(testUriPattern, getTestUser(), getTestPassword(), testHosts, getTestDatabase());
+        String testUri = String.format(testUriPatternPriorities, 60, 80, 100, 20, 40);
+        int[] hostsOrder = new int[] { 3, 2, 1, 5, 4 };
+
+        final ClientFactory cf = new ClientFactory();
+        Client client = cf.getClient(testUri, "{\"pooling\" : {\"enabled\" : true, \"maxSize\" : 10} }");
+
+        for (int i = 0; i < hostsOrder.length; i++) {
+            int h = hostsOrder[i];
+
+            Session testSession = client.getSession();
+            assertTrue(UnreliableSocketFactory.isConnected());
+            testSession.close(); // Pool this connection.
+
+            String lastHost = UnreliableSocketFactory.getHostFromLastConnection();
+            assertEquals(UnreliableSocketFactory.getHostConnectedStatus("host" + h), lastHost);
+            List<String> connectionAttempts = UnreliableSocketFactory.getHostsFromAllConnections();
+            assertEquals(i == 0 ? 1 : 2, connectionAttempts.size());
+            for (int a = 0; a < connectionAttempts.size() - 1; a++) {
+                assertEquals(UnreliableSocketFactory.getHostFailedStatus("host" + hostsOrder[i - 1 + a]), connectionAttempts.get(a));
+            }
+            UnreliableSocketFactory.downHost("host" + h);
+            UnreliableSocketFactory.flushConnectionAttempts();
+        }
+
+        // None of the hosts is available by now.
+        assertThrows(CJCommunicationsException.class, "Unable to connect to any of the target hosts\\.", () -> {
+            client.getSession();
+            return null;
+        });
+        // Final connection tried the last known to be good host (host4) and then the remaining hosts by their priority
+        List<String> connectionAttempts = UnreliableSocketFactory.getHostsFromAllConnections();
+        for (int i = 0; i < hostsOrder.length; i++) {
+            assertEquals(UnreliableSocketFactory.getHostFailedStatus("host" + hostsOrder[(i + 4) % 5]), connectionAttempts.get(i));
+        }
+
+        UnreliableSocketFactory.dontDownHost("host" + hostsOrder[1]);
+
+        UnreliableSocketFactory.flushConnectionAttempts();
+        Session testSession = client.getSession();
+        assertTrue(UnreliableSocketFactory.isConnected());
+        testSession.close();
+        connectionAttempts = UnreliableSocketFactory.getHostsFromAllConnections();
+        assertEquals(2, connectionAttempts.size());
+        assertEquals(UnreliableSocketFactory.getHostFailedStatus("host" + hostsOrder[0]), connectionAttempts.get(0));
+        assertEquals(UnreliableSocketFactory.getHostConnectedStatus("host" + hostsOrder[1]), connectionAttempts.get(1));
+
+        UnreliableSocketFactory.flushConnectionAttempts();
+        Session testSession1 = client.getSession(); // Pick previous connection from the pool. Doesn't count as new connections.
+        assertEquals(0, UnreliableSocketFactory.getHostsFromAllConnections().size());
+        Session testSession2 = client.getSession(); // Create a new connection.
+        assertTrue(UnreliableSocketFactory.isConnected());
+        testSession1.close();
+        testSession2.close();
+        connectionAttempts = UnreliableSocketFactory.getHostsFromAllConnections();
+        assertEquals(1, connectionAttempts.size());
+        assertEquals(UnreliableSocketFactory.getHostConnectedStatus("host" + hostsOrder[1]), connectionAttempts.get(0));
+
+        client.close();
+    }
+
+    @Test
+    public void pooledSessionFailoverByPrioritiesAndNoPooling() {
+        if (!this.isSetForXTests) {
+            return;
+        }
+        UnreliableSocketFactory.flushAllStaticData();
+
+        final String testUriPattern = "mysqlx://%s:%s@[%s]/%s?" + PropertyKey.xdevapiConnectTimeout.getKeyName() + "=100&"
+                + PropertyKey.socketFactory.getKeyName() + "=" + UnreliableSocketFactory.class.getName();
+        String testHosts = IntStream.range(1, 6).mapToObj(i -> "host" + i).peek(h -> UnreliableSocketFactory.mapHost(h, getTestHost()))
+                .map(h -> "(address=" + h + ":" + getTestPort() + ",priority=%d)").collect(Collectors.joining(","));
+        String testUriPatternPriorities = String.format(testUriPattern, getTestUser(), getTestPassword(), testHosts, getTestDatabase());
+        String testUri = String.format(testUriPatternPriorities, 60, 80, 100, 20, 40);
+        int[] hostsOrder = new int[] { 3, 2, 1, 5, 4 };
+
+        final ClientFactory cf = new ClientFactory();
+        Client client = cf.getClient(testUri, "{\"pooling\" : {\"enabled\" : true, \"maxSize\" : 10} }");
+
+        for (int i = 0; i < hostsOrder.length; i++) {
+            int h = hostsOrder[i];
+
+            client.getSession(); // Don't pool this connection.
+            assertTrue(UnreliableSocketFactory.isConnected());
+
+            String lastHost = UnreliableSocketFactory.getHostFromLastConnection();
+            assertEquals(UnreliableSocketFactory.getHostConnectedStatus("host" + h), lastHost);
+            List<String> connectionAttempts = UnreliableSocketFactory.getHostsFromAllConnections();
+            assertEquals(i == 0 ? 1 : 2, connectionAttempts.size());
+            for (int a = 0; a < connectionAttempts.size() - 1; a++) {
+                assertEquals(UnreliableSocketFactory.getHostFailedStatus("host" + hostsOrder[i - 1 + a]), connectionAttempts.get(a));
+            }
+            UnreliableSocketFactory.downHost("host" + h);
+            UnreliableSocketFactory.flushConnectionAttempts();
+        }
+
+        // None of the hosts is available by now.
+        assertThrows(CJCommunicationsException.class, "Unable to connect to any of the target hosts\\.", () -> {
+            client.getSession();
+            return null;
+        });
+        // Final connection tried the last known to be good host (host4) and then the remaining hosts by their priority
+        List<String> connectionAttempts = UnreliableSocketFactory.getHostsFromAllConnections();
+        for (int i = 0; i < hostsOrder.length; i++) {
+            assertEquals(UnreliableSocketFactory.getHostFailedStatus("host" + hostsOrder[(i + 4) % 5]), connectionAttempts.get(i));
+        }
+
+        UnreliableSocketFactory.dontDownHost("host" + hostsOrder[1]);
+
+        UnreliableSocketFactory.flushConnectionAttempts();
+        client.getSession();
+        assertTrue(UnreliableSocketFactory.isConnected());
+        connectionAttempts = UnreliableSocketFactory.getHostsFromAllConnections();
+        assertEquals(2, connectionAttempts.size());
+        assertEquals(UnreliableSocketFactory.getHostFailedStatus("host" + hostsOrder[0]), connectionAttempts.get(0));
+        assertEquals(UnreliableSocketFactory.getHostConnectedStatus("host" + hostsOrder[1]), connectionAttempts.get(1));
+
+        UnreliableSocketFactory.flushConnectionAttempts();
+        client.getSession();
+        assertTrue(UnreliableSocketFactory.isConnected());
+        connectionAttempts = UnreliableSocketFactory.getHostsFromAllConnections();
+        assertEquals(1, connectionAttempts.size());
+        assertEquals(UnreliableSocketFactory.getHostConnectedStatus("host" + hostsOrder[1]), connectionAttempts.get(0));
+
+        client.close();
+    }
 }
